# pio_wave_sym.py - 12.09.2021
#
# Sendet eine Anzahl von symmetrischen Impulsen mit festzulegender Frequenz
# Frequenzbereich von 200 Hz bis 12.5 MHz
# Die Anzahl der zu sendenden Waves wird vor Beginn ins X-Register kopiert.
# Soll die Anzahl nicht begrenzt werden, dann muss die letzte asm-Instruktion
# auf jmp("next") geändert werden. 

import time
from rp2 import PIO, StateMachine, asm_pio
from machine import Pin


@asm_pio(set_init=PIO.OUT_LOW, in_shiftdir=PIO.SHIFT_RIGHT)
def pulse_train_sym():
    
    wrap_target()
    
    pull(block)              #   1 warten auf Input(Anzahl der Pulse)
    mov(x, osr)              #   2 Anzahl nach x zum Downcounten
    
    label("next")            #     pulse + pause = 10 Takte 
    set(pins, 1)         [4] #   3 5 Takte
    set(pins, 0)         [3] #   4 1 + 3 + 1 = 5 Takte
    jmp(x_dec, "next")       #   5 bedingter jmp für begrenzte Wave
    #jmp("next")             #   5 alternativ für endlose Wave
    
    wrap()

class Wave_Sym(object):
    def __init__(self, statemachine, output_pin, wave_freq):
       self.sm = StateMachine(statemachine,
                        pulse_train_sym,
                        freq=wave_freq*10,
                        set_base=Pin(output_pin))
       self.sm.restart()
       
    def transmit(self, cnt):
        self.sm.active(1)
        self.sm.put(cnt - 1)

    def activate(self, state=1):
        self.sm.active(1 if state else 0)

if __name__ == "__main__":
    
    output_pin = 0
    wave_freq=1000
    wave_cnt = 7
    
    wav = Wave_Sym(0, output_pin=output_pin, wave_freq=wave_freq)
    wav.activate()
    wav.transmit(wave_cnt)
    
    time.sleep(.3)
    wav.activate(0)
    print("\done")
    

        
